@@ -29,10 +29,11 @@
  */
 
 #include "avcodec.h"
-#include "bytestream.h"
+#include "bswapdsp.h"
 #include "get_bits.h"
 #include "aandcttab.h"
 #include "eaidct.h"
+#include "idctdsp.h"
 #include "internal.h"
 #include "mpeg12.h"
 #include "mpeg12data.h"
@@ -45,7 +46,9 @@
 
 typedef struct MadContext {
     AVCodecContext *avctx;
-    DSPContext dsp;
+    BlockDSPContext bdsp;
+    BswapDSPContext bbdsp;
+    IDCTDSPContext idsp;
     AVFrame *last_frame;
     GetBitContext gb;
     void *bitstream_buf;
@@ -62,9 +65,11 @@ static av_cold int decode_init(AVCodecContext *avctx)
     MadContext *s = avctx->priv_data;
     s->avctx = avctx;
     avctx->pix_fmt = AV_PIX_FMT_YUV420P;
-    ff_dsputil_init(&s->dsp, avctx);
-    ff_init_scantable_permutation(s->dsp.idct_permutation, FF_NO_IDCT_PERM);
-    ff_init_scantable(s->dsp.idct_permutation, &s->scantable, ff_zigzag_direct);
+    ff_blockdsp_init(&s->bdsp, avctx);
+    ff_bswapdsp_init(&s->bbdsp);
+    ff_idctdsp_init(&s->idsp, avctx);
+    ff_init_scantable_permutation(s->idsp.idct_permutation, FF_NO_IDCT_PERM);
+    ff_init_scantable(s->idsp.idct_permutation, &s->scantable, ff_zigzag_direct);
     ff_mpeg12_init_vlcs();
 
     s->last_frame = av_frame_alloc();
@@ -145,11 +150,6 @@ static inline int decode_block_intra(MadContext *s, int16_t * block)
                 break;
             } else if (level != 0) {
                 i += run;
-                if (i > 63) {
-                    av_log(s->avctx, AV_LOG_ERROR,
-                           "ac-tex damaged at %d %d\n", s->mb_x, s->mb_y);
-                    return -1;
-                }
                 j = scantable[i];
                 level = (level*quant_matrix[j]) >> 4;
                 level = (level-1)|1;
@@ -164,11 +164,6 @@ static inline int decode_block_intra(MadContext *s, int16_t * block)
                 run = SHOW_UBITS(re, &s->gb, 6)+1; LAST_SKIP_BITS(re, &s->gb, 6);
 
                 i += run;
-                if (i > 63) {
-                    av_log(s->avctx, AV_LOG_ERROR,
-                           "ac-tex damaged at %d %d\n", s->mb_x, s->mb_y);
-                    return -1;
-                }
                 j = scantable[i];
                 if (level < 0) {
                     level = -level;
@@ -180,6 +175,10 @@ static inline int decode_block_intra(MadContext *s, int16_t * block)
                     level = (level-1)|1;
                 }
             }
+            if (i > 63) {
+                av_log(s->avctx, AV_LOG_ERROR, "ac-tex damaged at %d %d\n", s->mb_x, s->mb_y);
+                return -1;
+            }
 
             block[j] = level;
         }
@@ -202,7 +201,7 @@ static int decode_motion(GetBitContext *gb)
 static int decode_mb(MadContext *s, AVFrame *frame, int inter)
 {
     int mv_map = 0;
-    int mv_x, mv_y;
+    int av_uninit(mv_x), av_uninit(mv_y);
     int j;
 
     if (inter) {
@@ -220,7 +219,7 @@ static int decode_mb(MadContext *s, AVFrame *frame, int inter)
             if (s->last_frame->data[0])
                 comp_block(s, frame, s->mb_x, s->mb_y, j, mv_x, mv_y, add);
         } else {
-            s->dsp.clear_block(s->block);
+            s->bdsp.clear_block(s->block);
             if(decode_block_intra(s, s->block) < 0)
                 return -1;
             idct_put(s, frame, s->block, s->mb_x, s->mb_y, j);
@@ -244,33 +243,31 @@ static int decode_frame(AVCodecContext *avctx,
 {
     const uint8_t *buf = avpkt->data;
     int buf_size       = avpkt->size;
+    const uint8_t *buf_end = buf+buf_size;
     MadContext *s     = avctx->priv_data;
     AVFrame *frame    = data;
-    GetByteContext gb;
     int width, height;
     int chunk_type;
     int inter, ret;
 
-    bytestream2_init(&gb, buf, buf_size);
-
-    chunk_type = bytestream2_get_le32(&gb);
-    inter = (chunk_type == MADm_TAG || chunk_type == MADe_TAG);
-    bytestream2_skip(&gb, 10);
-
-    av_reduce(&avctx->time_base.num, &avctx->time_base.den,
-              bytestream2_get_le16(&gb), 1000, 1<<30);
-
-    width  = bytestream2_get_le16(&gb);
-    height = bytestream2_get_le16(&gb);
-    bytestream2_skip(&gb, 1);
-    calc_quant_matrix(s, bytestream2_get_byte(&gb));
-    bytestream2_skip(&gb, 2);
-
-    if (bytestream2_get_bytes_left(&gb) < 2) {
-        av_log(avctx, AV_LOG_ERROR, "Input data too small\n");
+    if (buf_size < 26) {
+        av_log(avctx, AV_LOG_ERROR, "Input buffer too small\n");
+        *got_frame = 0;
         return AVERROR_INVALIDDATA;
     }
 
+    chunk_type = AV_RL32(&buf[0]);
+    inter = (chunk_type == MADm_TAG || chunk_type == MADe_TAG);
+    buf += 8;
+
+    av_reduce(&avctx->time_base.num, &avctx->time_base.den,
+              AV_RL16(&buf[6]), 1000, 1<<30);
+
+    width  = AV_RL16(&buf[8]);
+    height = AV_RL16(&buf[10]);
+    calc_quant_matrix(s, buf[13]);
+    buf += 16;
+
     if (width < 16 || height < 16) {
         av_log(avctx, AV_LOG_ERROR, "Dimensions too small\n");
         return AVERROR_INVALIDDATA;
@@ -278,7 +275,7 @@ static int decode_frame(AVCodecContext *avctx,
 
     if (avctx->width != width || avctx->height != height) {
         av_frame_unref(s->last_frame);
-        if((width * height)/2048*7 > bytestream2_get_bytes_left(&gb))
+        if((width * height)/2048*7 > buf_end-buf)
             return AVERROR_INVALIDDATA;
         if ((ret = ff_set_dimensions(avctx, width, height)) < 0)
             return ret;
@@ -301,13 +298,13 @@ static int decode_frame(AVCodecContext *avctx,
     }
 
     av_fast_padded_malloc(&s->bitstream_buf, &s->bitstream_buf_size,
-                          bytestream2_get_bytes_left(&gb));
+                          buf_end - buf);
     if (!s->bitstream_buf)
         return AVERROR(ENOMEM);
-    s->dsp.bswap16_buf(s->bitstream_buf, (const uint16_t *)(buf + bytestream2_tell(&gb)),
-                         bytestream2_get_bytes_left(&gb) / 2);
-    memset((uint8_t*)s->bitstream_buf + bytestream2_get_bytes_left(&gb), 0, FF_INPUT_BUFFER_PADDING_SIZE);
-    init_get_bits(&s->gb, s->bitstream_buf, 8*(bytestream2_get_bytes_left(&gb)));
+    s->bbdsp.bswap16_buf(s->bitstream_buf, (const uint16_t *) buf,
+                         (buf_end - buf) / 2);
+    memset((uint8_t*)s->bitstream_buf + (buf_end-buf), 0, FF_INPUT_BUFFER_PADDING_SIZE);
+    init_get_bits(&s->gb, s->bitstream_buf, 8*(buf_end-buf));
 
     for (s->mb_y=0; s->mb_y < (avctx->height+15)/16; s->mb_y++)
         for (s->mb_x=0; s->mb_x < (avctx->width +15)/16; s->mb_x++)
