@@ -647,6 +647,9 @@ static int ogg_read_close(AVFormatContext *s)
         av_freep(&ogg->streams[i].private);
         av_freep(&ogg->streams[i].new_metadata);
     }
+
+    ogg->nstreams = 0;
+
     av_freep(&ogg->streams);
     return 0;
 }
@@ -793,11 +796,8 @@ retry:
         uint8_t *side_data = av_packet_new_side_data(pkt,
                                                      AV_PKT_DATA_SKIP_SAMPLES,
                                                      10);
-        if(side_data == NULL) {
-            av_free_packet(pkt);
-            av_free(pkt);
-            return AVERROR(ENOMEM);
-        }
+        if(side_data == NULL)
+            goto fail;
         AV_WL32(side_data + 4, os->end_trimming);
         os->end_trimming = 0;
     }
@@ -806,12 +806,18 @@ retry:
         uint8_t *side_data = av_packet_new_side_data(pkt,
                                                      AV_PKT_DATA_METADATA_UPDATE,
                                                      os->new_metadata_size);
+        if(side_data == NULL)
+            goto fail;
+
         memcpy(side_data, os->new_metadata, os->new_metadata_size);
         av_freep(&os->new_metadata);
         os->new_metadata_size = 0;
     }
 
     return psize;
+fail:
+    av_free_packet(pkt);
+    return AVERROR(ENOMEM);
 }
 
 static int64_t ogg_read_timestamp(AVFormatContext *s, int stream_index,
@@ -830,7 +836,7 @@ static int64_t ogg_read_timestamp(AVFormatContext *s, int stream_index,
            && !ogg_packet(s, &i, &pstart, &psize, pos_arg)) {
         if (i == stream_index) {
             struct ogg_stream *os = ogg->streams + stream_index;
-            // Dont trust the last timestamps of a ogm video
+            // Do not trust the last timestamps of a ogm video
             if (    (os->flags & OGG_FLAG_EOS)
                 && !(os->flags & OGG_FLAG_BOS)
                 && os->codec == &ff_ogm_video_codec)
